from __future__ import print_function
from joblib import Parallel, delayed
from optparse import OptionParser
import csv
import os
import re
import scipy as sp

# We assume that the vectors always have the same order.

parser = OptionParser(usage="Usage: %prog [options]")
parser.add_option("-s", "--space", type=str, default="sphere")
parser.add_option("--prefix", type=str, default="Outex_TC_00013")
parser.add_option('--project', action="store_true", default=False)
(options,_) = parser.parse_args()

spaceName = options.space
prefix = options.prefix
projectBack = options.project

if projectBack:
    spaceName = "%s-projected" % spaceName

texDirName = "%s/features/%s" % (prefix,spaceName)
classFileName = "%s/classification/%s.csv" % (prefix,spaceName)

try:
    os.makedirs(os.path.dirname(classFileName))
except OSError:
    # Should be okay, or otherwise we will fail later on anyway.
    # The reason to do this instead of checking whether the directory exists, is that the latter can lead to a race condition.
    pass

caseLinePattern = re.compile(r'(\d+)\.[a-z]+\s+(\d+)')

trainList = []
testList = []

with open("%s/000/train.txt" % prefix,"rt") as trainFile:
    trainN = int(trainFile.next())
    print("Training cases: %d" % trainN)
    for line in trainFile:
        m = re.match(caseLinePattern, line)
        trainList.append((m.group(1),int(m.group(2))))
assert(len(trainList)==trainN)

with open("%s/000/test.txt" % prefix,"rt") as testFile:
    testN = int(testFile.next())
    print("Test cases: %d" % testN)
    for line in testFile:
        m = re.match(caseLinePattern, line)
        testList.append((m.group(1),int(m.group(2))))
    pass
assert(len(testList)==testN)

featureFileCache = {}
def readFeatureFile(fname):
    if fname not in featureFileCache:
        with open(fname,"rt") as featureFile:
            reader = csv.reader(featureFile)
            reader.next()
            table = []
            for row in reader:
                table.append(map(float,row[3:103]))
        featureFileCache[fname] = sp.array(table)
    return featureFileCache[fname]

def matchTest(testId,testClass):
    testFeature = readFeatureFile("%s/%s.csv" % (texDirName,testId))
    minDist,minId,minClass = sp.inf,'',-1
    for trainId,trainClass in trainList:
        trainFeature = readFeatureFile("%s/%s.csv" % (texDirName,trainId))
        dist = sp.sqrt(sp.mean(sp.square(testFeature-trainFeature)))
        if dist<minDist:
            minDist = dist
            minId = trainId
            minClass = trainClass
    return (minId,minClass,minDist)    

matchList = Parallel(n_jobs=-2)(delayed(matchTest)(testId,testClass) for testId,testClass in testList)

matches = 0
with open(classFileName,"wb") as classFile:
    classFile.write('"image id","inferred class","real class","distance","match"\n')
    for testId,testClass,minId,minClass,minDist in map(lambda a,b:a+b, testList, matchList):
        classFile.write('%s,%d,%d,%.6g,%s\n' % (testId,minClass,testClass,minDist,minId))
        if testClass == minClass:
            matches += 1
print("Classification rate: %.2f" % (100.0*matches/len(testList)))

