from __future__ import print_function
from joblib import Parallel, delayed
from optparse import OptionParser
import csv
import os
import re
import scipy as sp

# We assume that the vectors always have the same order.

parser = OptionParser(usage="Usage: %prog [options]")
parser.add_option("--space1", type=str, default="basis")
parser.add_option("--space2", type=str, default="sphere")
parser.add_option("--prefix", type=str, default="Outex_TC_00013")
(options,_) = parser.parse_args()

space1Name = options.space1
space2Name = options.space2
prefix = options.prefix

tex1DirName = "%s/features/%s" % (prefix,space1Name)
tex2DirName = "%s/features/%s" % (prefix,space2Name)
classFileName = "%s/classification/%s vs %s.csv" % (prefix,space1Name,space2Name)

try:
    os.makedirs(os.path.dirname(classFileName))
except OSError:
    # Should be okay, or otherwise we will fail later on anyway.
    # The reason to do this instead of checking whether the directory exists, is that the latter can lead to a race condition.
    pass

caseLinePattern = re.compile(r'(\d+)\.[a-z]+\s+(\d+)')

trainList = []
testList = []

with open("%s/000/train.txt" % prefix,"rt") as trainFile:
    trainN = int(trainFile.next())
    print("Training cases: %d" % trainN)
    for line in trainFile:
        m = re.match(caseLinePattern, line)
        trainList.append((m.group(1),int(m.group(2))))
assert(len(trainList)==trainN)

with open("%s/000/test.txt" % prefix,"rt") as testFile:
    testN = int(testFile.next())
    print("Test cases: %d" % testN)
    for line in testFile:
        m = re.match(caseLinePattern, line)
        testList.append((m.group(1),int(m.group(2))))
    pass
assert(len(testList)==testN)

featureFileCache = {}
def readFeatureFile(fname):
    if fname not in featureFileCache:
        with open(fname,"rt") as featureFile:
            reader = csv.reader(featureFile)
            reader.next()
            table = []
            for row in reader:
                table.append(map(float,row[3:103]))
        featureFileCache[fname] = sp.array(table)
    return featureFileCache[fname]

def matchTest(testId,testClass):
    testFeature1 = readFeatureFile("%s/%s.csv" % (tex1DirName,testId))
    testFeature2 = readFeatureFile("%s/%s.csv" % (tex2DirName,testId))
    minDist1,minDist12,minId1,minClass1 = sp.inf,sp.inf,'',-1
    minDist2,minDist21,minId2,minClass2 = sp.inf,sp.inf,'',-1
    for trainId,trainClass in trainList:
        trainFeature1 = readFeatureFile("%s/%s.csv" % (tex1DirName,trainId))
        trainFeature2 = readFeatureFile("%s/%s.csv" % (tex2DirName,trainId))
        dist1 = sp.sqrt(sp.mean(sp.square(testFeature1-trainFeature1)))
        dist2 = sp.sqrt(sp.mean(sp.square(testFeature2-trainFeature2)))
        if dist1<minDist1:
            minDist1 = dist1
            minDist12 = dist2
            minId1 = trainId
            minClass1 = trainClass
        if dist2<minDist2:
            minDist2 = dist2
            minDist21 = dist1
            minId2 = trainId
            minClass2 = trainClass
    return (minId1,minClass1,minDist1,minDist12,minId2,minClass2,minDist2,minDist21)

matchList = Parallel(n_jobs=-2)(delayed(matchTest)(testId,testClass) for testId,testClass in testList)

matches1 = 0
matches2 = 0
with open(classFileName,"wb") as classFile:
    classFile.write('"image id","inferred class","","real class","test to match","","test to other match","","match",""\n')
    for testId,testClass,minId1,minClass1,minDist1,minDist12,minId2,minClass2,minDist2,minDist21 in map(lambda a,b:a+b, testList, matchList):
        classFile.write('%s,%d,%d,%d,%.6g,%.6g,%.6g,%.6g,%s,%s\n' % (testId,minClass1,minClass2,testClass,minDist1,minDist2,minDist12,minDist21,minId1,minId2))
        if testClass == minClass1:
            matches1 += 1
        if testClass == minClass2:
            matches2 += 1
print("Classification rates: %.2f, %.2f" % (100.0*matches1/len(testList),100.0*matches2/len(testList)))

