from __future__ import print_function
from operations import operations
from optparse import OptionParser
import os
import PIL.Image as Image
import re
import scipy as sp
import scipy.ndimage as ni

parser = OptionParser(usage="Usage: %prog [options] images")
parser.add_option("-o", "--operation", type="choice", default="occo-3x3-cross", choices=list(iter(operations(1).keys())))
parser.add_option("--scale", type=int, default=1)
parser.add_option("--out", type=str, default=None)
(options,args) = parser.parse_args()
if len(args) != 1:
    parser.error("Exactly one image required.")

inputFilename = args[0]
operation = operations(options.scale)[options.operation]

if options.out is None:
    outputFilename = '%s-out.png' % os.path.splitext(inputFilename)
else:
    outputFilename = options.out
basepath,extension = os.path.splitext(outputFilename)
basename = os.path.basename(basepath)

try:
    os.makedirs(os.path.dirname(outputFilename))
except OSError:
    # Should be okay, or otherwise we will fail later on anyway.
    # The reason to do this instead of checking whether the directory exists, is that the latter can lead to a race condition.
    pass

print("Processing %s" % basename)

I = sp.array(Image.open(inputFilename),dtype='uint32')
I = I[:,:,:3]

# This encodes RGB values in a 32 bit integer so that the normal order on the integers gives the lexicographical order on the RGB values.
# Using 1024 instead of 256 gives some headroom to make sure things like the OCCO filter work correctly.
# The mod call does use 256 because the results should be in the correct range, and we want to avoid getting in trouble due to a division by two of an odd number (which essentially spills a bit to the next value).
Il = I[:,:,0]*1024*1024 + I[:,:,1]*1024 + I[:,:,2]
Il = sp.trunc(operation(Il)).astype('uint32')
LI = sp.zeros(I.shape,dtype='uint32')
LI[:,:,0] = sp.mod(Il//(1024*1024),256)
LI[:,:,1] = sp.mod(Il//1024,256)
LI[:,:,2] = sp.mod(Il,256)

Image.fromarray(LI.astype('uint8')).save(outputFilename)

