import scipy as sp
import scipy.ndimage as ni

def footprintCross(r):
    dist1d = sp.absolute(range(-r,r+1))
    dist1d2d = sp.repeat([dist1d],2*r+1,axis=0)
    dist2d = dist1d2d + dist1d2d.T
    return dist2d<=r

def operations(scale):
    r3 = (3*scale)//2
    r5 = (5*scale)//2
    fpc3 = footprintCross(r3)
    return {
        "dilation-5x5": lambda G: ni.grey_dilation(G,size=(2*r5+1,2*r5+1)),
        "erosion-5x5": lambda G: ni.grey_erosion(G,size=(2*r5+1,2*r5+1)),
        "closing-5x5": lambda G: ni.grey_closing(G,size=(2*r5+1,2*r5+1)),
        "opening-5x5": lambda G: ni.grey_opening(G,size=(2*r5+1,2*r5+1)),
        "occo-5x5": lambda G: (ni.grey_opening(ni.grey_closing(G,size=(2*r5+1,2*r5+1)),size=(2*r5+1,2*r5+1))+ni.grey_closing(ni.grey_opening(G,size=(2*r5+1,2*r5+1)),size=(2*r5+1,2*r5+1)))/2,
        "occo-3x3": lambda G: (ni.grey_opening(ni.grey_closing(G,size=(2*r3+1,2*r3+1)),size=(2*r3+1,2*r3+1))+ni.grey_closing(ni.grey_opening(G,size=(2*r3+1,2*r3+1)),size=(2*r3+1,2*r3+1)))/2,
        "occo-3x3-cross": lambda G: (ni.grey_opening(ni.grey_closing(G,footprint=fpc3),footprint=fpc3)+ni.grey_closing(ni.grey_opening(G,footprint=fpc3),footprint=fpc3))/2,
        "co-3x3-cross": lambda G: ni.grey_closing(ni.grey_opening(G,footprint=fpc3),footprint=fpc3),
        "opening-3x3-cross": lambda G: ni.grey_opening(G,footprint=fpc3),
        "laplace": lambda G: ni.grey_dilation(G,size=(2*r3+1,2*r3+1))+ni.grey_erosion(G,size=(2*r3+1,2*r3+1))-2*G,
        "median-5x5": lambda G: ni.median_filter(G,size=(2*r5+1,2*r5+1)),
        "median-3x3-cross": lambda G: ni.median_filter(G,footprint=fpc3),
    }

